use std::fs;
use std::path;

use regex::Regex;

#[derive(Debug)]
pub struct Post {
    pub title: String,
    pub body: String,
    pub slug: String,
}

pub fn parse_post(path: path::PathBuf) -> Post {
    let contents = fs::read_to_string(&path).expect("Couldn't read post file");

    lazy_static! {
        static ref re: Regex = Regex::new(r"^# (?P<title>.*)\n\n(?P<body>.*)").unwrap();
        static ref slug_re: Regex = Regex::new(r"(?P<slug>\S+).md").unwrap();
    }

    let title = &re.captures(&contents).expect("Couldn't parse title")["title"];
    let body = &re.captures(&contents).expect("Couldn't parse body")["body"];

    let filename = &path.file_name().unwrap().to_str().unwrap();
    let slug = &slug_re.captures(filename).expect("Couldn't parse slug")["slug"];
    Post {
        title: String::from(title),
        body: String::from(body),
        slug: String::from(slug),
    }
}

mod tests {
    use super::*;
    use std::{env, fs};
    use uuid::Uuid;

    #[test]
    fn test_parse_post() {
        let temp_dir = env::temp_dir();
        let working_dir = temp_dir.join(&Uuid::new_v4().to_string());
        fs::create_dir(&working_dir).unwrap();
        env::set_current_dir(&working_dir).unwrap();

        let cwd = env::current_dir().unwrap();
        fs::create_dir(cwd.join("posts")).unwrap();

        let slug = Uuid::new_v4().to_string();
        let filename = format!("{}.md", slug);
        fs::write(
            cwd.join("posts").join(&filename),
            "# This is a post\n\nHere is some content that goes in the post",
        ).unwrap();

        let post = parse_post(cwd.join("posts").join(&filename));
        assert_eq!("This is a post", post.title);
        assert_eq!("Here is some content that goes in the post", post.body);
        assert_eq!(slug, post.slug);

        fs::remove_dir_all(temp_dir.join(&working_dir)).unwrap();
    }
}
